@@ -1,16 +1,18 @@
 # Copyright 2021-2022 Boris Shminke
-
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-
+#
 #     https://www.apache.org/licenses/LICENSE-2.0
-
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
+# noqa: D205, D400
 """
 Agent Testing
 ==============
@@ -29,6 +31,7 @@ import orjson
 from gym.wrappers import TimeLimit
 
 from gym_saturation.envs.saturation_env import (
+    MAX_CLAUSES,
     STATE_DIFF_UPDATED,
     SaturationEnv,
 )
@@ -39,13 +42,13 @@ from gym_saturation.logic_ops.utils import (
 
 
 class BaseAgent:
-    """a basic RL agent"""
+    """A basic RL agent."""
 
     _state = None
 
     @property
     def state(self):
-        """agent can have its inner state"""
+        """Agent can have its inner state."""
         return self._state
 
     def get_action(
@@ -55,6 +58,8 @@ class BaseAgent:
         info: Dict[str, Any],
     ) -> int:
         """
+        Get an action given observations and other inputs.
+
         :param observation: an observation returned by the environment after
             the latest action (or it's initial state)
         :param reward: reward from the previous step
@@ -67,8 +72,9 @@ class BaseAgent:
 @dataclass
 class Transition:
     """
-    an object describing environment's and agent's states
-    before the agent's action, the action itself and its results
+    An object describing environment's and agent's states.
+
+    Before the agent's action, the action itself and its results
     """
 
     env_state: dict
@@ -82,20 +88,20 @@ class Transition:
 
 class SizeAgent(BaseAgent):
     """
-    .. _size_agent:
+    Agent which selects the shortest clause.
 
-    agent which selects the shortest clause
+    .. _size_agent:
     """
 
     def __init__(self):
+        """Don't compute the formulae lengths twice."""
         self._state: Dict[str, Tuple[float, bool]] = {}
 
     def update_state(self, info: Dict[str, Any]) -> None:
         """
-        update the state of the agent according with the transition
+        Update the state of the agent according with the transition.
 
         :param info: an info dict (parm of environment response)
-        :returns:
         """
         parsed_state_diff = tuple(map(orjson.loads, info[STATE_DIFF_UPDATED]))
         self._state.update(
@@ -110,7 +116,7 @@ class SizeAgent(BaseAgent):
         observation: dict,
         reward: float,
         info: Dict[str, Any],
-    ) -> int:
+    ) -> int:  # noqa: D102
         self.update_state(info)
         return min(
             (
@@ -124,9 +130,9 @@ class SizeAgent(BaseAgent):
 
 class AgeAgent(BaseAgent):
     """
-    .. _age_agent:
+    Agent which selects the oldest clause.
 
-    agent which selects the oldest clause
+    .. _age_agent:
     """
 
     def get_action(
@@ -134,27 +140,30 @@ class AgeAgent(BaseAgent):
         observation: dict,
         reward: float,
         info: Dict[str, Any],
-    ) -> int:
+    ) -> int:  # noqa: D102
         return min(
-            [
-                i
-                for i, clause in enumerate(
-                    map(orjson.loads, observation["real_obs"])
-                )
-                if not clause["processed"]
-            ]
+            i
+            for i, clause in enumerate(
+                map(orjson.loads, observation["real_obs"])
+            )
+            if not clause["processed"]
         )
 
 
 class SizeAgeAgent(BaseAgent):
     """
-    .. _size_age_agent:
+    Agent taking several times the smallest clause and then the oldest.
 
-    agent which takes several times the smallest clause and then several
-    times the oldest
+    .. _size_age_agent:
     """
 
     def __init__(self, size_steps: int, age_steps: int):
+        """
+        Initialize two sub-agents.
+
+        :param size_steps: how many times to select the shortest clause
+        :param age_steps: how many times to select the oldest clause
+        """
         self.size_steps = size_steps
         self.age_steps = age_steps
         self._step_count = 0
@@ -167,7 +176,7 @@ class SizeAgeAgent(BaseAgent):
         observation: dict,
         reward: float,
         info: Dict[str, Any],
-    ) -> int:
+    ) -> int:  # noqa: D102
         self._step_count += 1
         if self._use_size:
             if self._step_count >= self.size_steps:
@@ -182,14 +191,14 @@ class SizeAgeAgent(BaseAgent):
 
 
 class RandomAgent(BaseAgent):
-    """agent which selects clauses randomly"""
+    """Agent which selects clauses randomly."""
 
     def get_action(
         self,
         observation: dict,
         reward: float,
         info: Dict[str, Any],
-    ) -> int:
+    ) -> int:  # noqa: D102
         return random.choice(
             [
                 i
@@ -209,9 +218,18 @@ def _proof_found_before_the_start(env: SaturationEnv) -> Tuple[float, bool]:
         return 0.0, False
 
 
+def _reset_with_options(env: gym.Env) -> dict:
+    res = env.reset()
+    if isinstance(res, tuple):
+        obs, _ = res  # pragma: nocover
+    else:
+        obs = res
+    return obs
+
+
 def episode(env: SaturationEnv, agent: BaseAgent) -> float:
     """
-    tries to solve the problem and logs the clauses
+    Try to solve the problem and logs the clauses.
 
     >>> import os
     >>> import shutil
@@ -233,11 +251,11 @@ def episode(env: SaturationEnv, agent: BaseAgent) -> float:
     >>> random.seed(0)
     >>> agents = (SizeAgeAgent(2, 1), SizeAgeAgent(1, 2), RandomAgent())
     >>> max_clauses = (100, 100, 4)
-    >>> for i in range(3):  # doctest: +ELLIPSIS
+    >>> for i in range(3):
     ...     env = gym.make(
     ...         "GymSaturation-v0",
     ...         problem_list=[problem_list[i]],
-    ...         max_clauses=max_clauses[i]
+    ...         max_clauses=max_clauses[i],
     ...     )
     ...     env._max_episode_steps = 5
     ...     agent_testing_report(env, agents[i])
@@ -248,10 +266,9 @@ def episode(env: SaturationEnv, agent: BaseAgent) -> float:
 
     :param env: a `gym_saturation` environment
     :param agent: an initialized agent. Must have `get_action` method
-    :param problem_filename: the name of a problem file
     :returns: total reward
     """
-    obs = env.reset()
+    obs = _reset_with_options(env)
     info: Dict[str, Any] = {STATE_DIFF_UPDATED: obs["real_obs"]}
     reward, done = _proof_found_before_the_start(env)
     total_reward = reward
@@ -264,17 +281,22 @@ def episode(env: SaturationEnv, agent: BaseAgent) -> float:
 
 def parse_args(args: Optional[List[str]] = None) -> Namespace:
     """
+    Parse script arguments.
+
     >>> parse_args([
-    ...     "--problem_filename", "test",
-    ...     "--step_limit", "1"
+    ...     "--problem_filename", "this_is_a_test_case",
+    ...     "--step_limit", "1",
     ... ])
-    Namespace(problem_filename='test', step_limit=1, vampire_binary_path=None)
+    Namespace(max_clauses=100000, problem_filename='this_is_a_test_case', step_limit=1, vampire_binary_path=None)
 
     :param args: a list of string arguments
         (for testing and use in a non script scenario)
     :returns: arguments namespace for the script
     """
     argument_parser = ArgumentParser()
+    argument_parser.add_argument(
+        "--max_clauses", type=int, required=False, default=MAX_CLAUSES
+    )
     argument_parser.add_argument("--problem_filename", type=str, required=True)
     argument_parser.add_argument("--step_limit", type=int, required=True)
     argument_parser.add_argument(
@@ -286,11 +308,10 @@ def parse_args(args: Optional[List[str]] = None) -> Namespace:
 
 def agent_testing_report(env: SaturationEnv, agent: BaseAgent) -> None:
     """
-        print a report after testing an agent in an environment
+    Print a report after testing an agent in an environment.
 
     :param env: an environment
     :param agent: an agent
-    :returns:
     """
     total_reward = episode(env, agent)
     step_count = getattr(env, "_elapsed_steps")
@@ -312,7 +333,7 @@ def agent_testing_report(env: SaturationEnv, agent: BaseAgent) -> None:
 
 def test_agent(args: Optional[List[str]] = None) -> None:
     """
-    the main function for this module
+    The main function for this module.
 
     >>> if sys.version_info.major == 3 and sys.version_info.minor >= 9:
     ...     from importlib.resources import files
@@ -329,7 +350,7 @@ def test_agent(args: Optional[List[str]] = None) -> None:
     >>> test_agent([
     ...     "--problem_filename", problem_filenames[0],
     ...     "--step_limit", "3"
-    ... ])  # doctest: +ELLIPSIS
+    ... ])
     Problem file: ...TST001-1.p
     Proof of length 1 found in 2 steps:
     cnf(..., lemma, $false, inference(resolution, [], [this_is_a_test_case_1, this_is_a_test_case_2])).
@@ -338,7 +359,7 @@ def test_agent(args: Optional[List[str]] = None) -> None:
     ...         "--problem_filename", problem_filename,
     ...         "--step_limit", "10",
     ...         "--vampire_binary_path", "vampire",
-    ...     ])  # doctest: +ELLIPSIS
+    ...     ])
     Problem file: ...TST001-1.p
     Proof of length 6 found in 0 steps:
     ...
@@ -349,23 +370,27 @@ def test_agent(args: Optional[List[str]] = None) -> None:
     Proof of length 5 found in 3 steps:
     ...
     cnf(5, hyp..., $false, inference(subsumption_resolution, [], [4, 3])).
+
+    :param args: parameters from the command line or explicitly set
     """
     sys.setrecursionlimit(10000)
     arguments = parse_args(args)
-    if arguments.vampire_binary_path is not None:
-        env = gym.make(
+    env = (
+        gym.make(
             "GymVampire-v0",
             problem_list=[arguments.problem_filename],
             vampire_binary_path=arguments.vampire_binary_path,
+            max_clauses=arguments.max_clauses,
         )
-    else:
-        env = gym.make(
+        if arguments.vampire_binary_path is not None
+        else gym.make(
             "GymSaturation-v0",
             problem_list=[arguments.problem_filename],
         )
+    )
     environment = TimeLimit(env, arguments.step_limit)
     print(f"Problem file: {arguments.problem_filename}")
-    agent_testing_report(environment, SizeAgeAgent(5, 1))
+    agent_testing_report(environment, SizeAgeAgent(5, 1))  # type: ignore
 
 
 if __name__ == "__main__":
