@@ -11,18 +11,21 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
+# noqa: D205, D400
 """
 Clause Space
 =============
 """
+import orjson
 from gym import spaces
 
 
 class ClauseSpace(spaces.Space):
     """
-    .. _clause_space:
+    An OpenAI Gym space for a list of bytes.
 
-    an OpenAI Gym space for a list of dictionaries
+    .. _clause_space:
 
     >>> space = ClauseSpace()
     >>> space.sample() in space
@@ -33,24 +36,26 @@ class ClauseSpace(spaces.Space):
     False
     """
 
-    def __init__(self):
+    def __init__(self):  # noqa: D107
         super().__init__(shape=(0,))
 
-    def contains(self, x):
+    def contains(self, x):  # noqa: D102
         if isinstance(x, list):
-            for clause in x:
-                return isinstance(clause, dict)
+            for byte_sequence in x:
+                return isinstance(byte_sequence, bytes)
         return False
 
-    def sample(self):
+    def sample(self):  # noqa: D102
         return [
-            {
-                "literals": (),
-                "label": "test",
-                "role": "lemma",
-                "birth_step": 0,
-                "processed": False,
-                "inference_parents": (),
-                "inference_rule": "",
-            }
+            orjson.dumps(
+                {
+                    "literals": (),
+                    "label": "test",
+                    "role": "lemma",
+                    "birth_step": 0,
+                    "processed": False,
+                    "inference_parents": (),
+                    "inference_rule": "",
+                }
+            )
         ]
