@@ -11,6 +11,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
+# noqa: D205, D400
 """
 Substitution
 =============
@@ -24,13 +26,24 @@ from tptp_lark_parser import grammar
 @dataclasses.dataclass
 class Substitution:
     """
-    a mapping from ``Variable`` to ``Term``
+    A mapping from ``Variable`` to ``Term``.
 
-    >>> substitution = Substitution(grammar.Variable("X"), grammar.Function("this_is_a_test_case", ()))
-    >>> substitution(grammar.Clause((grammar.Literal(False, grammar.Predicate("p", (grammar.Function("this_is_a_test_case", (grammar.Variable("X"),)),))),)))  # doctest: +ELLIPSIS
-    cnf(..., lemma, p(this_is_a_test_case(this_is_a_test_case))).
-    >>> substitution(grammar.Variable("X"))
-    Function(name='this_is_a_test_case', arguments=())
+    >>> substitution = Substitution(
+    ...     grammar.Variable(0), grammar.Function(2, ())
+    ... )
+    >>> substitution(
+    ...     grammar.Clause((
+    ...         grammar.Literal(
+    ...             False,
+    ...             grammar.Predicate(
+    ...                 3,
+    ...                 (grammar.Function(1, (grammar.Variable(0),)),))
+    ...             ),
+    ...     ), label="this_is_a_test_case")
+    ... )
+    Clause(literals=(Literal(negated=False, atom=Predicate(index=3, arguments=(Function(index=1, arguments=(Function(index=2, arguments=()),)),))),), label='this_is_a_test_case', role='lemma', inference_parents=None, inference_rule=None, processed=None, birth_step=None)
+    >>> substitution(grammar.Variable(0))
+    Function(index=2, arguments=())
     """
 
     variable: grammar.Variable
@@ -39,6 +52,11 @@ class Substitution:
     def __call__(
         self, target: Union[grammar.Clause, grammar.Proposition]
     ) -> Union[grammar.Clause, grammar.Proposition]:
+        """
+        Apply a substitution to a given target clause.
+
+        :param target: a clause to which to apply the substitution
+        """
         if isinstance(target, grammar.Clause):
             return self.substitute_in_clause(target)
         if isinstance(target, grammar.Predicate):
@@ -48,13 +66,13 @@ class Substitution:
     def _substitute_in_term(self, term: grammar.Term) -> grammar.Term:
         if isinstance(term, grammar.Function):
             return grammar.Function(
-                term.name,
+                term.index,
                 tuple(
                     self._substitute_in_term(argument)
                     for argument in term.arguments
                 ),
             )
-        if term.name == self.variable.name:
+        if term.index == self.variable.index:
             return self.term
         return term
 
@@ -62,7 +80,7 @@ class Substitution:
         self, predicate: grammar.Predicate
     ) -> grammar.Predicate:
         return grammar.Predicate(
-            predicate.name,
+            predicate.index,
             tuple(
                 self._substitute_in_term(argument)
                 for argument in predicate.arguments
@@ -71,7 +89,7 @@ class Substitution:
 
     def substitute_in_clause(self, clause: grammar.Clause) -> grammar.Clause:
         """
-        apply the substitution to something which is known to be a ``Clause``
+        Apply the substitution to something which is known to be a ``Clause``.
 
         :param clause: a clause to apply substitution to
         :returns: the result of the substitution
