@@ -11,15 +11,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
+# noqa: D205, D400
 """
 Logic Operations Utils
 =======================
 """
 import dataclasses
 from itertools import chain
-from typing import Dict, Tuple, Union
+from operator import attrgetter
+from typing import Any, Dict, Tuple, Union
 
 from tptp_lark_parser.grammar import (
+    EQUALITY_SYMBOL_ID,
     Clause,
     Function,
     Predicate,
@@ -32,14 +36,12 @@ from gym_saturation.logic_ops.substitution import Substitution
 
 
 class WrongRefutationProofError(Exception):
-    """
-    exception raised when proof is requested but is malformed or not found yet
-    """
+    """Exception raised when proof is requested but not found yet."""
 
 
 def is_subproposition(one: Proposition, two: Proposition) -> bool:
     """
-    check whether proposition ``one`` is part of a proposition ``two``
+    Check whether proposition ``one`` is part of a proposition ``two``.
 
     :param one: presumably shorter proposition which is probably a part
     :param two: presumably longer proposition which is probably the whole
@@ -58,11 +60,13 @@ def get_variable_list(
     clause: Union[Clause, Proposition]
 ) -> Tuple[Variable, ...]:
     """
-    find all variables present in a clause
+    Find all variables present in a clause.
 
     >>> from tptp_lark_parser.grammar import Literal
-    >>> get_variable_list(Clause((Literal(False, Predicate("this_is_a_test_case", (Function("f", (Variable("X"), Variable("X"))),))),)))
-    (Variable(name='X'), Variable(name='X'))
+    >>> get_variable_list(Clause((Literal(False,
+    ...     Predicate(0, (Function(0, (Variable(0), Variable(0))),))
+    ... ),)))
+    (Variable(index=0), Variable(index=0))
 
     :param clause: a clause
     :returns: a list (with repetitions) of variables from there clause
@@ -80,43 +84,76 @@ def get_variable_list(
     return variable_list
 
 
-def reindex_variables(
-    clauses: Dict[str, Clause], prefix: str
+def _shift_variables(
+    clauses: Dict[str, Clause], variable_list: Tuple[Variable, ...], shift: int
 ) -> Dict[str, Clause]:
-    """
-    rename variables in a list of clauses so that each clause has its unique
-    set of variables
-
-    :param clauses: a map of clause labels to clauses
-    :param prefix: new variables will be named ``prefix[order_num)``
-    :returns: the list of clauses with renamed variables
-    """
-    variable_count = 0
     new_clauses: Dict[str, Clause] = {}
     for label, clause in clauses.items():
         new_clause = clause
-        variable_list = tuple(set(get_variable_list(clause)))
-        new_variables_count = len(variable_list)
-        for i in range(new_variables_count):
+        for i, variable in enumerate(variable_list):
             new_clause = Substitution(
-                variable_list[i], Variable(f"{prefix}{i + variable_count}")
+                variable, Variable(shift + i)
             ).substitute_in_clause(new_clause)
-        variable_count += new_variables_count
         new_clauses[label] = new_clause
     return new_clauses
 
 
+def reindex_variables(clauses: Dict[str, Clause]) -> Dict[str, Clause]:
+    """
+    Rename variables so that each clause has its unique set of variables.
+
+    >>> from tptp_lark_parser.grammar import Literal
+    >>> clause = Clause((Literal(False,
+    ...     Predicate(0, (Variable(2), Variable(1), Variable(0)))
+    ... ),))
+    >>> sorted(map(
+    ...     attrgetter("index"),
+    ...     reindex_variables(
+    ...         {"this_is_a_test_case": clause}
+    ...     )["this_is_a_test_case"].literals[0].atom.arguments
+    ... ))
+    [0, 1, 2]
+    >>> clause = Clause((Literal(False,
+    ...     Predicate(0, (Variable(5), Variable(10), Variable(5)))
+    ... ),))
+    >>> sorted(map(
+    ...     attrgetter("index"),
+    ...     reindex_variables(
+    ...         {"this_is_a_test_case": clause}
+    ...     )["this_is_a_test_case"].literals[0].atom.arguments
+    ... ))
+    [0, 1, 1]
+
+    :param clauses: a map of clause labels to clauses
+    :returns: the list of clauses with renamed variables
+    """
+    variable_list = _flat_list(tuple(map(get_variable_list, clauses.values())))
+    shift = max(
+        len(variable_list),
+        1 + max(map(attrgetter("index"), variable_list), default=-1),
+    )
+    new_clauses = _shift_variables(clauses, variable_list, shift)
+    variable_list = _flat_list(
+        tuple(map(get_variable_list, new_clauses.values()))
+    )
+    new_clauses = _shift_variables(new_clauses, variable_list, 0)
+    return new_clauses
+
+
 def is_tautology(clause: Clause) -> bool:
     """
-    check whether there are two literals, one negated and the other not, with
-    the same atom
+    Check whether there are two literals (negated and not) with the same atom.
 
     >>> from tptp_lark_parser.grammar import Literal
-    >>> is_tautology(Clause((Literal(False, Predicate("this_is_a_test_case", ())),)))
+    >>> is_tautology(Clause((Literal(False, Predicate(7, ())),)))
     False
-    >>> is_tautology(Clause((Literal(False, Predicate("this_is_a_test_case", ())), Literal(True, Predicate("this_is_a_test_case", ())))))
+    >>> is_tautology(Clause(
+    ...     (Literal(False, Predicate(7, ())), Literal(True, Predicate(7, ())))
+    ... ))
     True
-    >>> is_tautology(Clause((Literal(False, Predicate("=", (Variable("X"), Variable("X")))),), label="this_is_a_test_case"))
+    >>> is_tautology(Clause(
+    ...     (Literal(False, Predicate(1, (Variable(0), Variable(0)))),)
+    ... ))
     True
 
     :param clause: a clause to check
@@ -130,7 +167,7 @@ def is_tautology(clause: Clause) -> bool:
                 and literal.atom == another_literal.atom
             ):
                 return True
-        if literal.atom.name == "=" and (
+        if literal.atom.index == EQUALITY_SYMBOL_ID and (
             literal.atom.arguments[0] == literal.atom.arguments[1]
         ):
             return True
@@ -139,8 +176,9 @@ def is_tautology(clause: Clause) -> bool:
 
 def clause_length(clause: dict) -> int:
     """
-    total length of arguments of each predicate.
-    Negation adds one to each literal
+    Find the length of arguments of each predicate.
+
+    Negation adds one to each literal.
 
     :param clause: a clause in JSON representation
     :return: sctructural length of a clause
@@ -155,7 +193,7 @@ def clause_length(clause: dict) -> int:
     length = 0
     if isinstance(clause, dict):
         for key, value in clause.items():
-            if key in {"negated", "name"}:
+            if key in {"negated", "index"}:
                 length += 1
             if isinstance(value, dict):
                 length += clause_length(value)
@@ -167,12 +205,12 @@ def clause_length(clause: dict) -> int:
 
 def proposition_length(proposition: Proposition) -> int:
     """
-    total number of functional, predicate and variable symbols
+    Find the number of functional, predicate and variable symbols.
 
     :param proposition: a function, a predicate or a variable
     :return: sctructural length of a proposition
 
-    >>> proposition_length(Predicate("p", (Function("f", (Variable("X"),)),)))
+    >>> proposition_length(Predicate(7, (Function(0, (Variable(0),)),)))
     3
     """
     length = 0
@@ -185,6 +223,7 @@ def proposition_length(proposition: Proposition) -> int:
 
 def clause_in_a_list(clause: Clause, clauses: Tuple[Clause, ...]) -> bool:
     """
+    Check whether a clause is in a list.
 
     >>> clause_in_a_list(Clause((), label="one"), (Clause((), label="two"),))
     True
@@ -198,19 +237,20 @@ def clause_in_a_list(clause: Clause, clauses: Tuple[Clause, ...]) -> bool:
 
 
 class NoSubtermFound(Exception):
-    """sometimes a subterm index is larger than term length"""
+    """Sometimes a subterm index is larger than term length."""
 
 
 def subterm_by_index(atom: Proposition, index: int) -> Term:
     """
-    extract a subterm using depth-first search through the tree of logical
-    operations
+    Extract a subterm using depth-first search.
 
-    >>> atom = Predicate("this_is_a_test_case", (Function("f", (Variable("X"),)), Function("g", (Variable("Y"),))))
+    >>> atom = Predicate(7, (
+    ...     Function(0, (Variable(0),)), Function(1, (Variable(1),))
+    ... ))
     >>> subterm_by_index(atom, 0)
     Traceback (most recent call last):
      ...
-    ValueError: subterm with index 0 exists only for terms, but got: Predicate(name='this_is_a_test_case', arguments=(Function(name='f', arguments=(Variable(name='X'),)), Function(name='g', arguments=(Variable(name='Y'),))))
+    ValueError: subterm with index 0 exists only for terms, but got: Predica...
     >>> subterm_by_index(atom, 1) == atom.arguments[0]
     True
     >>> subterm_by_index(atom, 2) == atom.arguments[0].arguments[0]
@@ -221,6 +261,8 @@ def subterm_by_index(atom: Proposition, index: int) -> Term:
     :param atom: a predicate or a term
     :param index: an index of a desired subterm
     :returns: a subterm
+    :raises ValueError: when trying to get a term with index 0 of a predicate
+    :raises NoSubtermFound: if subterm with a given index doesn't exist
     """
     if index == 0:
         if isinstance(atom, (Function, Variable)):
@@ -239,11 +281,11 @@ def subterm_by_index(atom: Proposition, index: int) -> Term:
 
 
 class CantReplaceTheWholeTerm(Exception):
-    """an exception raised when trying to replace a subterm with index 0"""
+    """An exception raised when trying to replace a subterm with index 0."""
 
 
 class TermSelfReplace(Exception):
-    """an exception raised when trying to replace a subterm with itself"""
+    """An exception raised when trying to replace a subterm with itself."""
 
 
 def _replace_if_not_the_same(old_term: Term, new_term: Term) -> Term:
@@ -256,16 +298,18 @@ def replace_subterm_by_index(
     atom: Proposition, index: int, term: Term
 ) -> Proposition:
     """
-    replace a subterm with a given index (depth-first search) by a new term
+    Replace a subterm with a given index (depth-first search) by a new term.
 
-    >>> atom = Predicate("this_is_a_test_case", (Function("f", (Variable("X"),)), Function("g", (Variable("Y"),))))
-    >>> replace_subterm_by_index(atom, 0, Variable("Z"))
+    >>> atom = Predicate(7, (
+    ...     Function(0, (Variable(0),)), Function(1, (Variable(2),))
+    ... ))
+    >>> replace_subterm_by_index(atom, 0, Variable(3))
     Traceback (most recent call last):
      ...
     gym_saturation.logic_ops.utils.NoSubtermFound: 5
-    >>> replace_subterm_by_index(atom, 4, Function("h", (Variable("Z"),)))
-    Predicate(name='this_is_a_test_case', arguments=(Function(name='f', arguments=(Variable(name='X'),)), Function(name='g', arguments=(Function(name='h', arguments=(Variable(name='Z'),)),))))
-    >>> replace_subterm_by_index(Predicate("this_is_a_test_case", (Variable("X"),)), 1, Variable("X"))
+    >>> "this_is_a_test_case", replace_subterm_by_index(atom, 4, Function(2, (Variable(3),)))
+    ('this_is_a_test_case', Predicate(index=7, arguments=(Function(index=0, arguments=(Variable(index=0),)), Function(index=1, arguments=(Function(index=2, arguments=(Variable(index=3),)),)))))
+    >>> replace_subterm_by_index(Predicate(7, (Variable(0),)), 1, Variable(0))
     Traceback (most recent call last):
      ...
     gym_saturation.logic_ops.utils.TermSelfReplace
@@ -274,6 +318,7 @@ def replace_subterm_by_index(
     :param index: an index of a subterm to replace, must be greater than 0
     :param term: replacement term for a given index
     :returns:
+    :raises NoSubtermFound: if subterm with a given index doesn't exist
     """
     subterm_length = 1
     if not isinstance(atom, Variable):
@@ -301,13 +346,13 @@ def replace_subterm_by_index(
     raise NoSubtermFound(subterm_length)
 
 
-def _flat_list(list_of_lists: Tuple[Tuple[str, ...], ...]) -> Tuple[str, ...]:
-    return tuple(reversed(sorted(tuple(set(chain(*list_of_lists))))))
+def _flat_list(list_of_lists: Tuple[Tuple[Any, ...], ...]) -> Tuple[Any, ...]:
+    return tuple(set(chain(*list_of_lists)))
 
 
 def reduce_to_proof(clauses: Dict[str, Clause]) -> Tuple[Clause, ...]:
     """
-    leave only clauses belonging to the refutational proof
+    Leave only clauses belonging to the refutational proof.
 
     >>> reduce_to_proof({
     ...     "one": Clause((), label="one"), "two": Clause((), label="two")
@@ -321,6 +366,8 @@ def reduce_to_proof(clauses: Dict[str, Clause]) -> Tuple[Clause, ...]:
 
     :param clauses: a map of clause labels to clauses
     :returns: the reduced list of clauses
+    :raises WrongRefutationProofError: if there is no complete refutation proof
+        in a given proof state
     """
     empty_clauses = tuple(
         clause for clause in clauses.values() if clause.literals == tuple()
@@ -334,14 +381,20 @@ def reduce_to_proof(clauses: Dict[str, Clause]) -> Tuple[Clause, ...]:
             )
             new_reduced = tuple(
                 clauses[label]
-                for label in _flat_list(
-                    tuple(
-                        (
-                            ()
-                            if clause.inference_parents is None
-                            else clause.inference_parents
+                for label in tuple(
+                    reversed(
+                        sorted(
+                            _flat_list(
+                                tuple(
+                                    (
+                                        ()
+                                        if clause.inference_parents is None
+                                        else clause.inference_parents
+                                    )
+                                    for clause in new_reduced
+                                )
+                            )
                         )
-                        for clause in new_reduced
                     )
                 )
             )
